package com.ruoyi.project.patrol.commonuse.service.impl;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.patrol.commonuse.mapper.TCommonuseBicycleMapper;
import com.ruoyi.project.patrol.commonuse.domain.TCommonuseBicycle;
import com.ruoyi.project.patrol.commonuse.service.ITCommonuseBicycleService;

/**
 * 自行车Service业务层处理
 * 
 * @author admin
 * @date 2023-11-09
 */
@Service
public class TCommonuseBicycleServiceImpl implements ITCommonuseBicycleService 
{
    @Autowired
    private TCommonuseBicycleMapper tCommonuseBicycleMapper;

    /**
     * 查询自行车
     * 
     * @param id 自行车主键
     * @return 自行车
     */
    @Override
    public TCommonuseBicycle selectTCommonuseBicycleById(Long id)
    {
        return tCommonuseBicycleMapper.selectTCommonuseBicycleById(id);
    }

    /**
     * 查询自行车列表
     * 
     * @param tCommonuseBicycle 自行车
     * @return 自行车
     */
    @Override
    public List<TCommonuseBicycle> selectTCommonuseBicycleList(TCommonuseBicycle tCommonuseBicycle)
    {
        return tCommonuseBicycleMapper.selectTCommonuseBicycleList(tCommonuseBicycle);
    }

    /**
     * 新增自行车
     * 
     * @param tCommonuseBicycle 自行车
     * @return 结果
     */
    @Override
    public int insertTCommonuseBicycle(TCommonuseBicycle tCommonuseBicycle)
    {
        return tCommonuseBicycleMapper.insertTCommonuseBicycle(tCommonuseBicycle);
    }

    /**
     * 修改自行车
     * 
     * @param tCommonuseBicycle 自行车
     * @return 结果
     */
    @Override
    public int updateTCommonuseBicycle(TCommonuseBicycle tCommonuseBicycle)
    {
        return tCommonuseBicycleMapper.updateTCommonuseBicycle(tCommonuseBicycle);
    }

    /**
     * 批量删除自行车
     * 
     * @param ids 需要删除的自行车主键
     * @return 结果
     */
    @Override
    public int deleteTCommonuseBicycleByIds(Long[] ids)
    {
        return tCommonuseBicycleMapper.deleteTCommonuseBicycleByIds(ids);
    }

    /**
     * 删除自行车信息
     * 
     * @param id 自行车主键
     * @return 结果
     */
    @Override
    public int deleteTCommonuseBicycleById(Long id)
    {
        return tCommonuseBicycleMapper.deleteTCommonuseBicycleById(id);
    }
}
